#!/usr/bin/env python

"""
geolocationtaskmenu.py

Task menu classes for the Qt Designer geolocation widget plugin.

Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
Contact: Qt Software Information (qt-info@nokia.com)

This file is part of the documentation of Qt. It was originally
published as part of Qt Quarterly.

Commercial Usage
Licensees holding valid Qt Commercial licenses may use this file in
accordance with the Qt Commercial License Agreement provided with the
Software or, alternatively, in accordance with the terms contained in
a written agreement between you and Nokia.


GNU General Public License Usage
Alternatively, this file may be used under the terms of the GNU
General Public License versions 2.0 or 3.0 as published by the Free
Software Foundation and appearing in the file LICENSE.GPL included in
the packaging of this file.  Please review the following information
to ensure GNU General Public Licensing requirements will be met:
http://www.fsf.org/licensing/licenses/info/GPLv2.html and
http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
exception, Nokia gives you certain additional rights. These rights
are described in the Nokia Qt GPL Exception version 1.3, included in
the file GPL_EXCEPTION.txt in this package.

Qt for Windows(R) Licensees
As a special exception, Nokia, as the sole copyright holder for Qt
Designer, grants users of the Qt/Eclipse Integration plug-in the
right for the Qt/Eclipse Integration to link to functionality
provided by Qt Designer and its related libraries.

If you are unsure which license is appropriate for your use, please
contact the sales department at qt-sales@nokia.com.
"""

from PyQt4.QtCore import QString, QVariant, SIGNAL, SLOT, pyqtSignature
from PyQt4.QtDesigner import QExtensionFactory, QPyDesignerTaskMenuExtension, \
                             QDesignerFormWindowInterface
from PyQt4.QtGui import QAction, QDialog, QDialogButtonBox, QGridLayout, \
                        QLineEdit, QPushButton
from QQ_Widgets.geolocationwidget import GeoLocationWidget


class GeoLocationMenuEntry(QPyDesignerTaskMenuExtension):

    """GeoLocationMenuEntry(QPyDesignerTaskMenuExtension)
    
    Provides a task menu entry to enable details of the geographical location
    to be edited in a dialog.
    """
    
    def __init__(self, widget, parent):
    
        QPyDesignerTaskMenuExtension.__init__(self, parent)
        
        self.widget = widget
        
        # Create the action to be added to the form's existing task menu
        # and connect it to a slot in this class.
        self.editStateAction = QAction(self.tr("Update Location..."), self)
        self.connect(self.editStateAction, SIGNAL("triggered()"),
                     self.updateLocation)
    
    def preferredEditAction(self):
        return self.editStateAction
    
    def taskActions(self):
        return [self.editStateAction]
    
    # The updateLocation() slot is called when the action that represents our
    # task menu entry is triggered. We open a dialog, passing the custom widget
    # as an argument.
    def updateLocation(self):
        dialog = GeoLocationDialog(self.widget)
        dialog.exec_()


class GeoLocationTaskMenuFactory(QExtensionFactory):

    """GeoLocationTaskMenuFactory(QExtensionFactory)
    
    Provides a task menu that can be used to access an editor dialog.
    """
    def __init__(self, parent = None):
    
        QExtensionFactory.__init__(self, parent)
    
    # This standard factory function returns an object to represent a task
    # menu entry.
    def createExtension(self, obj, iid, parent):
    
        if iid != "com.trolltech.Qt.Designer.TaskMenu":
            return None
        
        # We pass the instance of the custom widget to the object representing
        # the task menu entry so that the contents of the custom widget can be
        # modified.
        if isinstance(obj, GeoLocationWidget):
            return GeoLocationMenuEntry(obj, parent)
        
        return None


class GeoLocationDialog(QDialog):

    """GeoLocationDialog(QDialog)
    
    Provides a dialog that is used to edit the contents of the custom widget.
    """
    
    def __init__(self, widget, parent = None):
    
        QDialog.__init__(self, parent)
        
        # We keep a reference to the widget in the form.
        self.widget = widget
        
        self.previewWidget = GeoLocationWidget()
        self.previewWidget.latitude = widget.latitude
        self.previewWidget.longitude = widget.longitude
        
        buttonBox = QDialogButtonBox()
        okButton = buttonBox.addButton(buttonBox.Ok)
        cancelButton = buttonBox.addButton(buttonBox.Cancel)
        
        self.connect(okButton, SIGNAL("clicked()"),
                     self.updateWidget)
        self.connect(cancelButton, SIGNAL("clicked()"),
                     self, SLOT("reject()"))
        
        layout = QGridLayout()
        layout.addWidget(self.previewWidget, 1, 0, 1, 2)
        layout.addWidget(buttonBox, 2, 0, 1, 2)
        self.setLayout(layout)
        
        self.setWindowTitle(self.tr("Update Location"))
    
    # When we update the contents of the custom widget, we access its
    # properties via the QDesignerFormWindowInterface API so that Qt Designer
    # can integrate the changes we make into its undo-redo management.
    def updateWidget(self):
    
        formWindow = QDesignerFormWindowInterface.findFormWindow(self.widget)
        
        if formWindow:
            formWindow.cursor().setProperty("latitude",
                QVariant(self.previewWidget.latitude))
            formWindow.cursor().setProperty("longitude",
                QVariant(self.previewWidget.longitude))
        
        self.accept()
